@@ -98,24 +98,27 @@ module String = struct
       and len = length str in
       len >= sufflen && sub str (len - sufflen) sufflen = suffix
 
-    let rec find s sub =
-      let len = length s in
+    let find_from str pos sub =
       let sublen = length sub in
-      let rec loop i =
-        if i <= len-sublen then (
-          let rec loop2 j =
-            if j < sublen then (
-              if s.[i+j] = sub.[j] then loop2 (j+1)
-              else -1
-            ) else
-              i (* found *)
-          in
-          let r = loop2 0 in
-          if r = -1 then loop (i+1) else r
-        ) else
-          -1 (* not found *)
-      in
-      loop 0
+      if sublen = 0 then
+        0
+      else (
+        let found = ref 0 in
+        let len = length str in
+        try
+          for i = pos to len - sublen do
+            let j = ref 0 in
+            while unsafe_get str (i + !j) = unsafe_get sub !j do
+              incr j;
+              if !j = sublen then begin found := i; raise Exit; end;
+            done;
+          done;
+          -1
+        with
+          Exit -> !found
+      )
+
+    let find str sub = find_from str 0 sub
 
     let rec replace s s1 s2 =
       let len = length s in
@@ -139,29 +142,51 @@ module String = struct
       done;
       if not !r then s else Bytes.to_string b2
 
-    let rec split sep str =
-      let len = length sep in
-      let seplen = length str in
+    let break n str =
+      let len = String.length str in
+      if n < 0 then "", str
+      else if n >= len then str, ""
+      else sub str 0 n, sub str n (len-n)
+
+    let split sep str =
+      let seplen = length sep in
+      let strlen = length str in
       let i = find str sep in
       if i = -1 then str, ""
       else (
-        sub str 0 i, sub str (i + len) (seplen - i - len)
+        sub str 0 i, sub str (i + seplen) (strlen - i - seplen)
       )
 
-    and nsplit ?(max = 0) sep str =
+    let nsplit ?(max = 0) sep str =
       if max < 0 then
         invalid_arg "String.nsplit: max parameter should not be negative";
 
-      (* If we reached the limit, OR if the pattern does not match the string
-       * at all, return the rest of the string as a single element list.
-       *)
-      if max = 1 || find str sep = -1 then
-        [str]
-      else (
-        let s1, s2 = split sep str in
-        let max = if max = 0 then 0 else max - 1 in
-        s1 :: nsplit ~max sep s2
-      )
+      let len = String.length str in
+      let seplen = String.length sep in
+
+      let rec loop iters posn acc =
+        (* If we reached the limit, OR if the pattern does not match
+         * the string at all, return the rest of the string.
+         *)
+        if max > 0 && iters = max then (
+          let rest =
+            if posn = 0 then str else String.sub str posn (len-posn) in
+          List.rev (rest :: acc)
+        )
+        else (
+          let end_ = find_from str posn sep in
+          if end_ = -1 then (
+            let rest =
+              if posn = 0 then str else String.sub str posn (len-posn) in
+            List.rev (rest :: acc)
+          )
+          else (
+            let acc = String.sub str posn (end_-posn) :: acc in
+            loop (iters+1) (end_+seplen) acc
+          )
+        )
+      in
+      loop 1 0 []
 
     let rec lines_split str =
       let buf = Buffer.create 16 in
@@ -287,6 +312,25 @@ module List = struct
       | x :: xs when f x -> x :: takewhile f xs
       | _ -> []
 
+    let take n xs =
+      if n <= 0 then []
+      else (
+        (* This optimisation avoids copying xs. *)
+        let len = List.length xs in
+        if len <= n then xs
+        else (
+          let rec take n = function
+            | x :: xs when n >= 1 -> x :: take (n-1) xs
+            | _ -> []
+          in
+          take n xs
+        )
+      )
+    let rec drop n xs =
+      if n <= 0 then xs
+      else if xs = [] then []
+      else drop (n-1) (List.tl xs)
+
     let rec filter_map f = function
       | [] -> []
       | x :: xs ->
@@ -301,18 +345,27 @@ module List = struct
           | Some y -> y
           | None -> find_map f xs
 
+    let rec group_by = function
+      | [] -> []
+      | (day1, x1) :: (day2, x2) :: rest when day1 = day2 ->
+         let rest = group_by ((day2, x2) :: rest) in
+         let day, xs = List.hd rest in
+         (day, x1 :: xs) :: List.tl rest
+      | (day, x) :: rest ->
+         (day, [x]) :: group_by rest
+
     let rec combine3 xs ys zs =
       match xs, ys, zs with
       | [], [], [] -> []
       | x::xs, y::ys, z::zs -> (x, y, z) :: combine3 xs ys zs
       | _ -> invalid_arg "combine3"
 
-    let rec assoc_lbl ?(cmp = Pervasives.compare) ~default x = function
+    let rec assoc_lbl ?(cmp = Stdlib.compare) ~default x = function
       | [] -> default
       | (y, y') :: _ when cmp x y = 0 -> y'
       | _ :: ys -> assoc_lbl ~cmp ~default x ys
 
-    let uniq ?(cmp = Pervasives.compare) xs =
+    let uniq ?(cmp = Stdlib.compare) xs =
       let rec loop acc = function
         | [] -> acc
         | [x] -> x :: acc
@@ -323,14 +376,6 @@ module List = struct
       in
       List.rev (loop [] xs)
 
-    (* This is present in OCaml 4.04, so we can remove it when
-     * we depend on OCaml >= 4.04.
-     *)
-    let sort_uniq ?(cmp = Pervasives.compare) xs =
-      let xs = List.sort cmp xs in
-      let xs = uniq ~cmp xs in
-      xs
-
     let remove_duplicates xs =
       let h = Hashtbl.create (List.length xs) in
       let rec loop = function
@@ -357,24 +402,15 @@ module List = struct
       xsp := xs;
       x
 
+    let may_push_back xsp x =
+      match x with None -> () | Some x -> push_back xsp x
+    let may_push_front x xsp =
+      match x with None -> () | Some x -> push_front x xsp
+
     let push_back_list xsp xs = xsp := !xsp @ xs
     let push_front_list xs xsp = xsp := xs @ !xsp
 end
 
-module Option = struct
-    let may f = function
-      | None -> ()
-      | Some x -> f x
-
-    let map f = function
-      | None -> None
-      | Some x -> Some (f x)
-
-    let default def = function
-      | None -> def
-      | Some x -> x
-end
-
 let (//) = Filename.concat
 let quote = Filename.quote
 
@@ -544,41 +580,7 @@ let be64_of_int i =
   Bytes.unsafe_set b 7 (Char.unsafe_chr (Int64.to_int c0));
   Bytes.to_string b
 
-type wrap_break_t = WrapEOS | WrapSpace | WrapNL
-
-let rec wrap ?(chan = stdout) ?(indent = 0) str =
-  let len = String.length str in
-  _wrap chan indent 0 0 len str
-
-and _wrap chan indent column i len str =
-  if i < len then (
-    let (j, break) = _wrap_find_next_break i len str in
-    let next_column =
-      if column + (j-i) >= 76 then (
-        output_char chan '\n';
-        output_spaces chan indent;
-        indent + (j-i) + 1
-      )
-      else column + (j-i) + 1 in
-    output chan (Bytes.of_string str) i (j-i);
-    match break with
-    | WrapEOS -> ()
-    | WrapSpace ->
-      output_char chan ' ';
-      _wrap chan indent next_column (j+1) len str
-    | WrapNL ->
-      output_char chan '\n';
-      output_spaces chan indent;
-      _wrap chan indent indent (j+1) len str
-  )
-
-and _wrap_find_next_break i len str =
-  if i >= len then (len, WrapEOS)
-  else if String.unsafe_get str i = ' ' then (i, WrapSpace)
-  else if String.unsafe_get str i = '\n' then (i, WrapNL)
-  else _wrap_find_next_break (i+1) len str
-
-and output_spaces chan n = for i = 0 to n-1 do output_char chan ' ' done
+let output_spaces chan n = for i = 0 to n-1 do output_char chan ' ' done
 
 let unique = let i = ref 0 in fun () -> incr i; !i
 
@@ -591,21 +593,10 @@ let protect ~f ~finally =
   finally ();
   match r with Either ret -> ret | Or exn -> raise exn
 
-type 'a return = { return: 'b. 'a -> 'b } (* OCaml >= 4.03: [@@unboxed] *)
-(* This requires features in OCaml >= 4.04:
+type 'a return = { return: 'b. 'a -> 'b } [@@unboxed]
 let with_return (type a) f =
   let exception Return of a in
   try f {return = fun ret -> raise (Return ret)} with Return ret -> ret
-*)
-
-(* This should work for any version of OCaml, but it doesn't work
- * properly for nested with_return statements.  When we can assume
- * OCaml >= 4.04 we should use the above definition instead.
- *)
-let with_return f =
-  let ret = ref None in
-  try f {return = fun r -> ret := Some r; raise Exit}
-  with Exit -> match !ret with None -> assert false | Some r -> r
 
 let failwithf fs = ksprintf failwith fs
 
@@ -633,8 +624,8 @@ let which executable =
 (* Program name. *)
 let prog = Filename.basename Sys.executable_name
 
-(* Stores the colours (--colours), quiet (--quiet), trace (-x) and
- * verbose (-v) flags in a global variable.
+(* Stores the colours (--colours), quiet (--quiet), trace (-x), verbose (-v)
+ * and wrap (--wrap) flags in a global variable.
  *)
 let colours = ref false
 let set_colours () = colours := true
@@ -652,6 +643,10 @@ let verbose = ref false
 let set_verbose () = verbose := true
 let verbose () = !verbose
 
+let wrap = ref false
+let set_wrap () = wrap := true
+let wrap () = !wrap
+
 let with_open_in filename f =
   let chan = open_in filename in
   protect ~f:(fun () -> f chan) ~finally:(fun () -> close_in chan)
@@ -738,27 +733,6 @@ let stringify_args args =
   | [] -> ""
   | app :: xs -> app ^ quote_args xs
 
-(* Unlink a temporary file on exit. *)
-let unlink_on_exit =
-  let files = ref [] in
-  let registered_handlers = ref false in
-
-  let rec unlink_files () =
-    List.iter (
-      fun file -> try Unix.unlink file with _ -> ()
-    ) !files
-  and register_handlers () =
-    (* Unlink on exit. *)
-    at_exit unlink_files
-  in
-
-  fun file ->
-    files := file :: !files;
-    if not !registered_handlers then (
-      register_handlers ();
-      registered_handlers := true
-    )
-
 let is_block_device file =
   try (Unix.stat file).Unix.st_kind = Unix.S_BLK
   with Unix.Unix_error _ -> false
