@@ -82,6 +82,10 @@ module String : sig
     val find : string -> string -> int
     (** [find str sub] searches for [sub] as a substring of [str].  If
         found it returns the index.  If not found, it returns [-1]. *)
+    val find_from : string -> int -> string -> int
+    (** [find_from str start sub] searches for [sub] as a substring of [str],
+        starting at index [start].  If found it returns the index.
+        If not found, it returns [-1]. *)
     val replace : string -> string -> string -> string
     (** [replace str s1 s2] replaces all instances of [s1] appearing in
         [str] with [s2]. *)
@@ -290,21 +294,6 @@ module List : sig
 end
 (** Override the List module from stdlib. *)
 
-module Option : sig
-    val iter : ('a -> unit) -> 'a option -> unit
-    (** [iter f o] is [f v] if [o] is [Some v] and [()] otherwise *)
-
-    val map : ('a -> 'b) -> 'a option -> 'b option
-    (** [map f (Some x)] returns [Some (f x)].  [map f None] returns [None]. *)
-
-    val value : 'a option -> default:'a -> 'a
-    (** [value o ~default] is [v] if [o] is [Some v] and [default] otherwise. *)
-end
-(** Functions for dealing with option types.
-
-    This module will be removed when we can use baseline OCaml 4.08
-    since that version introduces a compatible [Option] module. *)
-
 val ( // ) : string -> string -> string
 (** Concatenate directory and filename. *)
 
