@@ -1,5 +1,5 @@
 (* virt-v2v
- * Copyright (C) 2009-2020 Red Hat Inc.
+ * Copyright (C) 2009-2025 Red Hat Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -131,6 +131,25 @@ let convert (g : G.guestfs) source inspect i_firmware _ keep_serial_console _ =
   (*----------------------------------------------------------------------*)
   (* Conversion step. *)
 
+  let uninstall_packages pkgs =
+    if pkgs <> [] then (
+      let cmd =
+        try Guest_packages.uninstall_command pkgs inspect.i_package_management
+        with
+        | Guest_packages.Unknown_package_manager msg
+        | Guest_packages.Unimplemented_package_manager msg ->
+          error "%s" msg
+      in
+      (try ignore (g#sh cmd)
+       with G.Error msg ->
+         warning (f_"could not uninstall packages ‘%s’: %s (ignored)")
+           (String.concat " " pkgs) msg
+      );
+      (* Reload Augeas in case anything changed. *)
+      Linux.augeas_reload g
+    )
+  in
+
   let rec do_convert () =
     augeas_grub_configuration ();
 
@@ -228,57 +247,6 @@ let convert (g : G.guestfs) source inspect i_firmware _ keep_serial_console _ =
       Linux.augeas_reload g
 
   and unconfigure_xen () =
-    (* Remove kmod-xenpv-* (RHEL 3). *)
-    let xenmods =
-      List.filter_map (
-        fun { G.app2_name = name } ->
-          if name = "kmod-xenpv" || String.is_prefix name "kmod-xenpv-" then
-            Some name
-          else
-            None
-      ) inspect.i_apps in
-    Linux.remove g inspect.i_root xenmods;
-
-    (* Undo related nastiness if kmod-xenpv was installed. *)
-    if xenmods <> [] then (
-      (* kmod-xenpv modules may have been manually copied to other kernels.
-       * Hunt them down and destroy them.
-       *)
-      let dirs = g#find "/lib/modules" in
-      let dirs = Array.to_list dirs in
-      let dirs = List.filter (fun s -> String.find s "/xenpv" >= 0) dirs in
-      let dirs = List.map ((^) "/lib/modules/") dirs in
-      let dirs = List.filter g#is_dir dirs in
-
-      (* Check it's not owned by an installed application. *)
-      let dirs = List.filter (
-        fun d -> not (Linux.is_file_owned g inspect.i_root d)
-      ) dirs in
-
-      (* Remove any unowned xenpv directories. *)
-      List.iter g#rm_rf dirs;
-
-      (* rc.local may contain an insmod or modprobe of the xen-vbd driver,
-       * added by an installation script.
-       *)
-      (try
-         let lines = g#read_lines "/etc/rc.local" in
-         let lines = Array.to_list lines in
-         let rex = PCRE.compile "\\b(insmod|modprobe)\\b.*\\bxen-vbd" in
-         let lines = List.map (
-           fun s ->
-             if PCRE.matches rex s then
-               "#" ^ s
-             else
-               s
-         ) lines in
-         let file = String.concat "\n" lines ^ "\n" in
-         g#write "/etc/rc.local" file
-       with
-         G.Error msg -> eprintf "%s: /etc/rc.local: %s (ignored)\n" prog msg
-      );
-    );
-
     if family = `SUSE_family then (
       (* Remove xen modules from INITRD_MODULES and DOMU_INITRD_MODULES. *)
       let variables = ["INITRD_MODULES"; "DOMU_INITRD_MODULES"] in
@@ -310,7 +278,7 @@ let convert (g : G.guestfs) source inspect i_firmware _ keep_serial_console _ =
         fun { G.app2_name = name } -> name = package_name
       ) inspect.i_apps in
     if has_guest_additions then
-      Linux.remove g inspect.i_root [package_name];
+      uninstall_packages [package_name];
 
     (* Guest Additions might have been installed from a tarball.  The
      * above code won't detect this case.  Look for the uninstall tool
@@ -455,8 +423,7 @@ let convert (g : G.guestfs) source inspect i_firmware _ keep_serial_console _ =
       )
     );
 
-    let remove = !remove in
-    Linux.remove g inspect.i_root remove;
+    uninstall_packages !remove;
 
     (* VMware Tools may have been installed from a tarball, so the
      * above code won't remove it.  Look for the uninstall tool and run
@@ -503,7 +470,7 @@ let convert (g : G.guestfs) source inspect i_firmware _ keep_serial_console _ =
     let pkgs = List.map (fun { G.app2_name = name } -> name) pkgs in
 
     if pkgs <> [] then (
-      Linux.remove g inspect.i_root pkgs;
+      uninstall_packages pkgs;
 
       (* Installing these guest utilities automatically unconfigures
        * ttys in /etc/inittab if the system uses it. We need to put
@@ -858,27 +825,6 @@ let convert (g : G.guestfs) source inspect i_firmware _ keep_serial_console _ =
          *)
         (try g#modprobe "loop" with G.Error _ -> ());
 
-        (* On RHEL 3 we have to take extra gritty to get a working
-         * loopdev.  mkinitrd runs the nash command `findlodev'
-         * which does this:
-         *
-         * for (devNum = 0; devNum < 256; devNum++) {
-         *   sprintf(devName, "/dev/loop%s%d", separator, devNum);
-         *   if ((fd = open(devName, O_RDONLY)) < 0) return 0;
-         *   if (ioctl(fd, LOOP_GET_STATUS, &loopInfo)) {
-         *     close(fd);
-         *     printf("%s\n", devName);
-         *     return 0;
-         * // etc
-         *
-         * In a modern kernel, /dev/loop<N> isn't created until it is
-         * used.  But we can create /dev/loop0 manually.  Note we have
-         * to do this in the appliance /dev.  (RHBZ#1171130)
-         *)
-        if family = `RHEL_family && inspect.i_major_version = 3 then
-          ignore (g#debug "sh" [| "mknod"; "-m"; "0666";
-                                  "/dev/loop0"; "b"; "7"; "0" |]);
-
         (* RHEL 4 mkinitrd determines if the root filesystem is on LVM
          * by checking if the device name (after following symlinks)
          * starts with /dev/mapper. However, on recent kernels/udevs,
