@@ -1,5 +1,5 @@
 (* virt-v2v
- * Copyright (C) 2009-2020 Red Hat Inc.
+ * Copyright (C) 2009-2025 Red Hat Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -56,18 +56,18 @@ let iso_time =
 
 (* Guess vmtype based on the guest inspection data. *)
 let get_vmtype = function
-  (* Special cases for RHEL 3 & RHEL 4. *)
-  | { i_type = "linux"; i_distro = ("rhel"|"centos"); i_major_version = (3|4);
+  (* Special cases for RHEL 4. *)
+  | { i_type = "linux"; i_distro = ("rhel"|"centos"); i_major_version = 4;
       i_product_name = product }
        when String.find product "ES" >= 0 ->
      `Server
 
-  | { i_type = "linux"; i_distro = ("rhel"|"centos"); i_major_version = (3|4);
+  | { i_type = "linux"; i_distro = ("rhel"|"centos"); i_major_version = 4;
       i_product_name = product }
        when String.find product "AS" >= 0 ->
      `Server
 
-  | { i_type = "linux"; i_distro = ("rhel"|"centos"); i_major_version = (3|4) } ->
+  | { i_type = "linux"; i_distro = ("rhel"|"centos"); i_major_version = 4 } ->
      `Desktop
 
   (* For Windows (and maybe Linux in future, but it is not set now),
@@ -242,10 +242,11 @@ and get_ostype = function
       | "win2k16" -> "windows_2016x64"
       | "win2k19" -> "windows_2019x64"
       | "win2k22" -> "windows_2022"
+      | "win2k25" -> "windows_2025"
       | _ ->
          warning (f_"unknown Windows 10 variant: %s (%s)")
            osinfo product;
-         "windows_2022"
+         "windows_2025"
      )
 
   | { i_type = typ; i_distro = distro;
@@ -262,14 +263,6 @@ and get_ostype = function
  *   https://bugzilla.redhat.com/show_bug.cgi?id=1219857#c9
  *)
 and get_ovirt_osid = function
-  | { i_type = "linux"; i_distro = ("rhel"|"centos"); i_major_version = 3;
-      i_arch = "i386" } ->
-    9
-
-  | { i_type = "linux"; i_distro = ("rhel"|"centos"); i_major_version = 3;
-      i_arch = "x86_64" } ->
-    15
-
   | { i_type = "linux"; i_distro = ("rhel"|"centos"); i_major_version = 4;
       i_arch = "i386" } ->
     8
@@ -448,6 +441,7 @@ and get_ovirt_osid = function
       | "win2k16" -> (* windows_2016x64 *) 29
       | "win2k19" -> (* windows_2019x64 *) 31
       | "win2k22" -> (* windows_2022 *) 37
+      (*| "win2k25" -> (* windows_2025 *) not yet known - 2024/10 *)
       | _ ->
          warning (f_"unknown Windows 10 variant: %s (%s)")
            osinfo product;
