@@ -1,5 +1,5 @@
 (* virt-v2v
- * Copyright (C) 2009-2024 Red Hat Inc.
+ * Copyright (C) 2009-2025 Red Hat Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,6 +29,12 @@ let large_tmpdir =
   try Sys.getenv "VIRT_V2V_TMPDIR"
   with Not_found -> (open_guestfs ())#get_cachedir ()
 
+let string_of_process_status = function
+  | Unix.WEXITED 0 -> s_"success"
+  | WEXITED i -> sprintf (f_"exited with non-zero error code %d") i
+  | WSIGNALED i -> sprintf (f_"signalled by signal %d") i
+  | WSTOPPED i -> sprintf (f_"stopped by signal %d") i
+
 (* Is SELinux enabled and enforcing on the host? *)
 let have_selinux =
   0 = Sys.command "getenforce 2>/dev/null | grep -isq Enforcing"
@@ -68,26 +74,6 @@ let qemu_supports_sound_card = function
   | Types.USBAudio
     -> true
 
-(* Find the UEFI firmware. *)
-let find_uefi_firmware guest_arch =
-  let files =
-    (* The lists of firmware are actually defined in common/utils/uefi.c. *)
-    match guest_arch with
-    | "x86_64" -> Uefi.uefi_x86_64_firmware
-    | "aarch64" -> Uefi.uefi_aarch64_firmware
-    | arch ->
-       error (f_"don’t know how to convert UEFI guests for architecture %s")
-             guest_arch in
-  let rec loop = function
-    | [] ->
-       error (f_"cannot find firmware for UEFI guests.\n\nYou probably \
-                 need to install OVMF (x86-64), or AAVMF (aarch64)")
-    | ({ Uefi.code; vars = vars_template } as ret) :: rest ->
-       if Sys.file_exists code && Sys.file_exists vars_template then ret
-       else loop rest
-  in
-  loop files
-
 let compare_app2_versions app1 app2 =
   let i = compare app1.Guestfs.app2_epoch app2.Guestfs.app2_epoch in
   if i <> 0 then i
