@@ -1,5 +1,5 @@
 (* virt-v2v
- * Copyright (C) 2009-2024 Red Hat Inc.
+ * Copyright (C) 2009-2025 Red Hat Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,6 +23,10 @@ val large_tmpdir : string
     such as overlays in this directory.  Small temporary files can
     use the default behaviour eg. of {!Filename.temp_file} *)
 
+val string_of_process_status : Unix.process_status -> string
+(** Convert a process status (such as returned by {!Unix.wait}) into
+    a printable string. *)
+
 val have_selinux : bool
 (** True if SELinux is enabled and enforcing on the host. *)
 
@@ -36,10 +40,6 @@ val kvm_arch : string -> string
 val qemu_supports_sound_card : Types.source_sound_model -> bool
 (** Does qemu support the given sound card? *)
 
-val find_uefi_firmware : string -> Uefi.uefi_firmware
-(** Find the UEFI firmware for the guest architecture.
-    This cannot return an error, it calls [error] and fails instead. *)
-
 val compare_app2_versions : Guestfs.application2 -> Guestfs.application2 -> int
 (** Compare two app versions. *)
 
