@@ -1,5 +1,5 @@
 (* virt-v2v
- * Copyright (C) 2009-2024 Red Hat Inc.
+ * Copyright (C) 2009-2025 Red Hat Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -258,11 +258,31 @@ let create_libvirt_xml ?pool source inspect
   let os_section =
     let os = ref [] in
 
-    let firmware =
+    let firmware_attribute =
       match target_firmware with
       | TargetBIOS -> []
       | TargetUEFI -> [ "firmware", "efi" ] in
 
+    let () =
+      match target_firmware with
+      | TargetBIOS -> ()
+      | TargetUEFI ->
+         (* The UEFI secureboot setting is sort of adjacent to if
+          * the full secure boot feature is enabled, but it's what
+          * we have.
+          * https://libvirt.org/kbase/secureboot.html
+          *)
+         let sb = source.s_uefi_secureboot in
+         let enrolled_keys =
+           if sb then
+             [e "feature" ["name", "enrolled-keys"; "enabled", "yes"] []]
+           else [] in
+         let firmware_features =
+           (e "feature" ["name", "secure-boot";
+                         "enabled", if sb then "yes" else "no"] []) ::
+             enrolled_keys in
+         List.push_back os (e "firmware" [] firmware_features) in
+
     let machine =
       match guestcaps.gcaps_machine with
       | I440FX -> "pc"
@@ -274,11 +294,7 @@ let create_libvirt_xml ?pool source inspect
                               "machine", machine]
                       [PCData "hvm"]);
 
-    (* We used to sometimes put <loader secure='yes'/> here.  However
-     * that is likely wrong since by its nature virt-v2v can't really
-     * support secure boot ever.  So for now we omit it.
-     *)
-    e "os" firmware !os in
+    e "os" firmware_attribute !os in
 
   (* The <clock> section. *)
   let clock_section =
